@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2022, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -110,6 +110,8 @@ TEST_CASE("1D Bilinear Mass Integrators",
                SparseMatrix * blfT = Transpose(blfw.SpMat());
                SparseMatrix * diff = Add(1.0, blf.SpMat(), -1.0, *blfT);
                REQUIRE( diff->MaxNorm() < tol );
+               delete diff;
+               delete blfT;
             }
             SECTION("With Coefficient")
             {
@@ -131,6 +133,8 @@ TEST_CASE("1D Bilinear Mass Integrators",
                SparseMatrix * blfT = Transpose(blfw.SpMat());
                SparseMatrix * diff = Add(1.0, blf.SpMat(), -1.0, *blfT);
                REQUIRE( diff->MaxNorm() < tol );
+               delete diff;
+               delete blfT;
             }
          }
       }
@@ -268,6 +272,8 @@ TEST_CASE("1D Bilinear Mass Integrators",
                SparseMatrix * blfT = Transpose(blfw.SpMat());
                SparseMatrix * diff = Add(1.0, blf.SpMat(), -1.0, *blfT);
                REQUIRE( diff->MaxNorm() < tol );
+               delete diff;
+               delete blfT;
             }
             SECTION("With Coefficient")
             {
@@ -289,6 +295,8 @@ TEST_CASE("1D Bilinear Mass Integrators",
                SparseMatrix * blfT = Transpose(blfw.SpMat());
                SparseMatrix * diff = Add(1.0, blf.SpMat(), -1.0, *blfT);
                REQUIRE( diff->MaxNorm() < tol );
+               delete diff;
+               delete blfT;
             }
          }
       }
@@ -466,6 +474,8 @@ TEST_CASE("1D Bilinear Weak Derivative Integrator",
             SparseMatrix * blfT = Transpose(blfw.SpMat());
             SparseMatrix * diff = Add(1.0, blf.SpMat(), 1.0, *blfT);
             REQUIRE( diff->MaxNorm() < tol );
+            delete diff;
+            delete blfT;
          }
          SECTION("With Coefficient")
          {
@@ -493,6 +503,8 @@ TEST_CASE("1D Bilinear Weak Derivative Integrator",
             SparseMatrix * blfT = Transpose(blfw.SpMat());
             SparseMatrix * diff = Add(1.0, blf.SpMat(), 1.0, *blfT);
             REQUIRE( diff->MaxNorm() < tol );
+            delete diff;
+            delete blfT;
          }
       }
    }
@@ -548,6 +560,8 @@ TEST_CASE("1D Bilinear Weak Derivative Integrator",
                SparseMatrix * blfT = Transpose(blfw.SpMat());
                SparseMatrix * diff = Add(1.0, blf.SpMat(), 1.0, *blfT);
                REQUIRE( diff->MaxNorm() < tol );
+               delete diff;
+               delete blfT;
             }
             SECTION("With Coefficient")
             {
@@ -575,6 +589,8 @@ TEST_CASE("1D Bilinear Weak Derivative Integrator",
                SparseMatrix * blfT = Transpose(blfw.SpMat());
                SparseMatrix * diff = Add(1.0, blf.SpMat(), 1.0, *blfT);
                REQUIRE( diff->MaxNorm() < tol );
+               delete diff;
+               delete blfT;
             }
          }
       }
