@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2022, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -19,37 +19,37 @@ using namespace mfem;
 
 namespace testhelper
 {
-double SmoothSolutionX(const mfem::Vector& x)
+real_t SmoothSolutionX(const mfem::Vector& x)
 {
    return x(0);
 }
 
-double SmoothSolutionY(const mfem::Vector& x)
+real_t SmoothSolutionY(const mfem::Vector& x)
 {
    return x(1);
 }
 
-double SmoothSolutionZ(const mfem::Vector& x)
+real_t SmoothSolutionZ(const mfem::Vector& x)
 {
    return x(2);
 }
 
-double NonsmoothSolutionX(const mfem::Vector& x)
+real_t NonsmoothSolutionX(const mfem::Vector& x)
 {
    return std::abs(x(0)-0.5);
 }
 
-double NonsmoothSolutionY(const mfem::Vector& x)
+real_t NonsmoothSolutionY(const mfem::Vector& x)
 {
    return std::abs(x(1)-0.5);
 }
 
-double NonsmoothSolutionZ(const mfem::Vector& x)
+real_t NonsmoothSolutionZ(const mfem::Vector& x)
 {
    return std::abs(x(2)-0.5);
 }
 
-double SinXSinY(const mfem::Vector& x)
+real_t SinXSinY(const mfem::Vector& x)
 {
    return std::sin(M_PI*x(0)) * std::sin(M_PI*x(1));
 }
@@ -164,9 +164,9 @@ TEST_CASE("Convergence rate test on 2D NCMesh", "[NCMesh]")
    // Define the solution vector x as a finite element grid function
    GridFunction x(&fespace);
 
-   double old_error = 0.0;
-   double old_num_dofs = 0.0;
-   double rate = 0.0;
+   real_t old_error = 0.0;
+   real_t old_num_dofs = 0.0;
+   real_t rate = 0.0;
    for (int it = 0; it < 4; it++)
    {
       int num_dofs = fespace.GetTrueVSize();
@@ -194,14 +194,14 @@ TEST_CASE("Convergence rate test on 2D NCMesh", "[NCMesh]")
 
       LSZienkiewiczZhuEstimator estimator(di, x);
       estimator.GetLocalErrors();
-      double error = estimator.GetTotalError();
+      real_t error = estimator.GetTotalError();
 
       if (old_error > 0.0)
       {
          rate = log(error/old_error) / log(old_num_dofs/num_dofs);
       }
 
-      old_num_dofs = double(num_dofs);
+      old_num_dofs = real_t(num_dofs);
       old_error = error;
 
       mesh.UniformRefinement();
@@ -244,7 +244,7 @@ TEST_CASE("Least-squares ZZ estimator on 3D NCMesh", "[NCMesh]")
       {
          REQUIRE(local_errors(i) < 1e-10);
       }
-      REQUIRE(estimator.GetTotalError() < 1e-10);;
+      REQUIRE(estimator.GetTotalError() < 1e-10);
    }
 
    SECTION("Perfect Approximation Y")
@@ -443,7 +443,7 @@ TEST_CASE("Kelly Error Estimator on 2D NCMesh embedded in 3D",
    const auto order = GENERATE(1, 3, 5);
 
    // Manually construct embedded mesh
-   std::array<double, 4*3> vertices =
+   std::array<real_t, 4*3> vertices =
    {
       0.0,0.0,0.0,
       0.0,1.0,0.0,
