@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2022, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -9,6 +9,10 @@
 // terms of the BSD-3 license. We welcome feedback and contributions, see file
 // CONTRIBUTING.md for details.
 
+#define CATCH_CONFIG_RUNNER
+#include "mfem.hpp"
+#include "run_unit_tests.hpp"
+
 #ifdef _WIN32
 #define _USE_MATH_DEFINES
 #include <cmath>
@@ -17,9 +21,6 @@
 #include <list>
 #include <fstream>
 #include <iostream>
-
-#include "mfem.hpp"
-#include "unit_tests.hpp"
 #include "miniapps/meshing/mesh-optimizer.hpp"
 
 #if defined(MFEM_TMOP_MPI) && !defined(MFEM_USE_MPI)
@@ -53,11 +54,11 @@ namespace mfem
 
 struct Req
 {
-   double init_energy;
-   double tauval;
-   double dot;
-   double final_energy;
-   double diag;
+   real_t init_energy;
+   real_t tauval;
+   real_t dot;
+   real_t final_energy;
+   real_t diag;
 };
 
 int tmop(int id, Req &res, int argc, char *argv[])
@@ -71,12 +72,13 @@ int tmop(int id, Req &res, int argc, char *argv[])
    int quad_type         = 1;
    int quad_order        = 2;
    int newton_iter       = 100;
-   double newton_rtol    = 1e-8;
+   real_t newton_rtol    = 1e-8;
    int lin_solver        = 2;
    int max_lin_iter      = 100;
-   double lim_const      = 0.0;
+   real_t lim_const      = 0.0;
+   int lim_type          = 0;
    int normalization     = 0;
-   double jitter         = 0.0;
+   real_t jitter         = 0.0;
    bool diag             = true;
    int newton_loop       = 1;
    int combo             = 0;
@@ -104,6 +106,7 @@ int tmop(int id, Req &res, int argc, char *argv[])
    args.AddOption(&lin_solver, "-ls", "--lin-solver", "");
    args.AddOption(&max_lin_iter, "-li", "--lin-iter", "");
    args.AddOption(&lim_const, "-lc", "--limit-const", "");
+   args.AddOption(&lim_type, "-lt", "--limit-type", "");
    args.AddOption(&normalization, "-nor", "--normalization", "");
    args.AddOption(&pa, "-pa", "--pa", "-no-pa", "--no-pa", "");
    args.AddOption(&jitter, "-ji", "--jitter", "");
@@ -132,18 +135,18 @@ int tmop(int id, Req &res, int argc, char *argv[])
    REQUIRE(order > 0);
    H1_FECollection fec(order, dim);
    ParFiniteElementSpace fes(pmesh, &fec, dim);
-   ParGridFunction x0(&fes), x(&fes);
+   ParGridFunction x0(&fes), x(&fes), x0_before_jitter(&fes);
    pmesh->SetNodalGridFunction(&x);
 
    Vector h0(fes.GetNDofs());
    h0 = infinity();
-   double volume = 0.0;
+   real_t volume = 0.0;
    {
       Array<int> dofs;
       for (int i = 0; i < pmesh->GetNE(); i++)
       {
          fes.GetElementDofs(i, dofs);
-         const double hi = pmesh->GetElementSize(i);
+         const real_t hi = pmesh->GetElementSize(i);
          for (int j = 0; j < dofs.Size(); j++)
          {
             h0(dofs[j]) = min(h0(dofs[j]), hi);
@@ -151,7 +154,11 @@ int tmop(int id, Req &res, int argc, char *argv[])
          volume += pmesh->GetElementVolume(i);
       }
    }
-   const double small_phys_size = pow(volume, 1.0 / dim) / 100.0;
+   const real_t small_phys_size = pow(volume, 1.0 / dim) / 100.0;
+
+   // When the target is GIVEN_SHAPE_AND_SIZE, we want to call tc->SetNodes()
+   // with something other than x0 (otherwise all metrics would be 0).
+   x0_before_jitter = x;
 
    ParGridFunction rdm(&fes);
    rdm.Randomize(seed);
@@ -185,11 +192,14 @@ int tmop(int id, Req &res, int argc, char *argv[])
       case   7: metric = new TMOP_Metric_007; break;
       case  77: metric = new TMOP_Metric_077; break;
       case  80: metric = new TMOP_Metric_080(0.5); break;
+      case  94: metric = new TMOP_Metric_094; break;
       case 302: metric = new TMOP_Metric_302; break;
       case 303: metric = new TMOP_Metric_303; break;
       case 315: metric = new TMOP_Metric_315; break;
+      case 318: metric = new TMOP_Metric_318; break;
       case 321: metric = new TMOP_Metric_321; break;
       case 332: metric = new TMOP_Metric_332(0.5); break;
+      case 338: metric = new TMOP_Metric_338; break;
       default:
       {
          if (id == 0) { cout << "Unknown metric_id: " << metric_id << endl; }
@@ -228,16 +238,7 @@ int tmop(int id, Req &res, int argc, char *argv[])
          target_t = TargetConstructor::IDEAL_SHAPE_GIVEN_SIZE;
          DiscreteAdaptTC *tc = new DiscreteAdaptTC(target_t);
          tc->SetAdaptivityEvaluator(new AdvectorCG(al));
-         if (dim == 2)
-         {
-            FunctionCoefficient ind_coeff(discrete_size_2d);
-            size.ProjectCoefficient(ind_coeff);
-         }
-         else if (dim == 3)
-         {
-            FunctionCoefficient ind_coeff(discrete_size_3d);
-            size.ProjectCoefficient(ind_coeff);
-         }
+         ConstructSizeGF(size);
          tc->SetDiscreteTargetSize(size);
          target_c = tc;
          break;
@@ -253,6 +254,10 @@ int tmop(int id, Req &res, int argc, char *argv[])
          target_c = tc;
          break;
       }
+      case 8: // fully specified through the initial mesh, 2D or 3D.
+      {
+         target_t = TargetConstructor::GIVEN_SHAPE_AND_SIZE; break;
+      }
       default:
       {
          if (id == 0) { cout << "Unknown target_id: " << target_id << endl; }
@@ -270,7 +275,7 @@ int tmop(int id, Req &res, int argc, char *argv[])
       target_c = new TargetConstructor(target_t);
    }
 #endif
-   target_c->SetNodes(x0);
+   target_c->SetNodes(x0_before_jitter);
 
    // Setup the quadrature rule for the non-linear form integrator.
    const IntegrationRule *ir = nullptr;
@@ -298,20 +303,33 @@ int tmop(int id, Req &res, int argc, char *argv[])
    ParGridFunction dist(&dist_fes);
    dist = 1.0;
    if (normalization == 1) { dist = small_phys_size; }
-   ConstantCoefficient lim_coeff(lim_const);
-   if (lim_const != 0.0) { he_nlf_integ->EnableLimiting(x0, dist, lim_coeff); }
+   auto coeff_lim_func = [&](const Vector &x) { return x(0) + lim_const; };
+   FunctionCoefficient lim_coeff(coeff_lim_func);
+   if (lim_const != 0.0)
+   {
+      if (lim_type == 0)
+      {
+         he_nlf_integ->EnableLimiting(x0, dist, lim_coeff);
+      }
+      else
+      {
+         he_nlf_integ->EnableLimiting(x0, dist, lim_coeff,
+                                      new TMOP_ExponentialLimiter);
+      }
+   }
 
    ParNonlinearForm nlf(&fes);
    nlf.SetAssemblyLevel(pa ? AssemblyLevel::PARTIAL : AssemblyLevel::LEGACY);
 
-   ConstantCoefficient *coeff1 = nullptr;
+   FunctionCoefficient *coeff1 = nullptr;
    TMOP_QualityMetric *metric2 = nullptr;
    TargetConstructor *target_c2 = nullptr;
    FunctionCoefficient coeff2(weight_fun);
    if (combo > 0)
    {
       // First metric.
-      coeff1 = new ConstantCoefficient(1.0);
+      auto coeff_1_func = [&](const Vector &x) { return x(0) + 10.0; };
+      coeff1 = new FunctionCoefficient(coeff_1_func);
       he_nlf_integ->SetCoefficient(*coeff1);
       // Second metric.
       if (dim == 2) { metric2 = new TMOP_Metric_077; }
@@ -344,7 +362,7 @@ int tmop(int id, Req &res, int argc, char *argv[])
    }
    nlf.Setup();
 
-   const double init_energy = nlf.GetParGridFunctionEnergy(x);
+   const real_t init_energy = nlf.GetParGridFunctionEnergy(x);
    res.init_energy = init_energy;
 
    // Fix all boundary nodes (-fix-bnd)
@@ -366,7 +384,18 @@ int tmop(int id, Req &res, int argc, char *argv[])
          TMOP_Integrator *nlfi_fa = new TMOP_Integrator(metric, target_c);
          nlfi_fa->SetIntegrationRule(*ir);
          if (normalization == 1) { nlfi_fa->ParEnableNormalization(x0); }
-         if (lim_const != 0.0) { nlfi_fa->EnableLimiting(x0, dist, lim_coeff); }
+         if (lim_const != 0.0)
+         {
+            if (lim_type == 0)
+            {
+               nlfi_fa->EnableLimiting(x0, dist, lim_coeff);
+            }
+            else
+            {
+               nlfi_fa->EnableLimiting(x0, dist, lim_coeff,
+                                       new TMOP_ExponentialLimiter);
+            }
+         }
          nlf_fa.AddDomainIntegrator(nlfi_fa);
          nlf_fa.SetEssentialBC(ess_bdr);
          dynamic_cast<GradientClass&>(nlf_fa.GetGradient(xt)).GetDiag(d);
@@ -376,7 +405,7 @@ int tmop(int id, Req &res, int argc, char *argv[])
 
    // Linear solver for the system's Jacobian
    Solver *S = nullptr, *S_prec = nullptr;
-   constexpr double linsol_rtol = 1e-12;
+   constexpr real_t linsol_rtol = 1e-12;
    if (lin_solver == 0)
    {
       S = new DSmoother(1, 1.0, max_lin_iter);
@@ -421,7 +450,7 @@ int tmop(int id, Req &res, int argc, char *argv[])
    }
 
    // Compute the minimum det(J) of the starting mesh
-   double tauval = infinity();
+   real_t tauval = infinity();
    const int NE = pmesh->GetNE();
    for (int i = 0; i < NE; i++)
    {
@@ -432,13 +461,15 @@ int tmop(int id, Req &res, int argc, char *argv[])
          tauval = min(tauval, transf->Jacobian().Det());
       }
    }
-   double minJ0;
-   MPI_Allreduce(&tauval, &minJ0, 1, MPI_DOUBLE, MPI_MIN, MPI_COMM_WORLD);
+   real_t minJ0;
+   MPI_Allreduce(&tauval, &minJ0, 1, MPITypeMap<real_t>::mpi_type, MPI_MIN,
+                 MPI_COMM_WORLD);
    tauval = minJ0;
    //if (id == 0) { cout << "Min det(J) of the mesh is " << tauval << endl; }
    REQUIRE(tauval > 0.0);
-   double h0min = h0.Min(), h0min_all;
-   MPI_Allreduce(&h0min, &h0min_all, 1, MPI_DOUBLE, MPI_MIN, MPI_COMM_WORLD);
+   real_t h0min = h0.Min(), h0min_all;
+   MPI_Allreduce(&h0min, &h0min_all, 1, MPITypeMap<real_t>::mpi_type, MPI_MIN,
+                 MPI_COMM_WORLD);
    tauval -= 0.01 * h0min_all; // Slightly below minJ0 to avoid div by 0.
    res.tauval = tauval;
 
@@ -470,7 +501,18 @@ int tmop(int id, Req &res, int argc, char *argv[])
       dist *= 0.93;
       if (normalization == 1) { dist = small_phys_size; }
 
-      if (lim_const != 0.0) { he_nlf_integ->EnableLimiting(x0, dist, lim_coeff); }
+      if (lim_const != 0.0)
+      {
+         if (lim_type == 0)
+         {
+            he_nlf_integ->EnableLimiting(x0, dist, lim_coeff);
+         }
+         else
+         {
+            he_nlf_integ->EnableLimiting(x0, dist, lim_coeff,
+                                         new TMOP_ExponentialLimiter);
+         }
+      }
 
       if (normalization == 1) { he_nlf_integ->ParEnableNormalization(x); }
 
@@ -481,13 +523,14 @@ int tmop(int id, Req &res, int argc, char *argv[])
 
       REQUIRE(newton->GetConverged());
 
-      const double final_energy = nlf.GetParGridFunctionEnergy(x);
+      const real_t final_energy = nlf.GetParGridFunctionEnergy(x);
       res.final_energy = final_energy;
    }
 
    Vector &x_t(x.GetTrueVector());
-   double x_t_dot = x_t*x_t, dot;
-   MPI_Allreduce(&x_t_dot, &dot, 1, MPI_DOUBLE, MPI_SUM, pmesh->GetComm());
+   real_t x_t_dot = x_t*x_t, dot;
+   MPI_Allreduce(&x_t_dot, &dot, 1, MPITypeMap<real_t>::mpi_type, MPI_SUM,
+                 pmesh->GetComm());
    res.dot = dot;
 
    delete S;
@@ -519,7 +562,8 @@ static void req_tmop(int id, const char *args[], Req &res)
     "tmop_tests", "-pa", "-m", "mesh", "-o", "0", "-rs", "0", \
     "-mid", "0", "-tid", "0", "-qt", "1", "-qo", "0", \
     "-ni", "10", "-rtol", "1e-8", "-ls", "2", "-li", "100", \
-    "-lc", "0", "-nor", "0", "-ji", "0", "-nl", "1", "-cmb", "0", nullptr }
+   "-lc", "0", "-nor", "0", "-ji", "0", "-nl", "1",\
+   "-cmb", "0", "-lt", "0", nullptr }
 constexpr int ALV = 1;
 constexpr int MSH = 3;
 constexpr int POR = 5;
@@ -536,6 +580,7 @@ constexpr int NOR = 27;
 constexpr int JI  = 29;
 constexpr int NL  = 31;
 constexpr int CMB = 33;
+constexpr int LT  = 35;
 
 static void dump_args(int id, const char *args[])
 {
@@ -543,7 +588,7 @@ static void dump_args(int id, const char *args[])
    const char *format =
       "tmop -m %s -o %s -qo %s -mid %s -tid %s -ls %s"
       "%s%s%s%s"         // Optional args: RS, QTY
-      "%s%s%s%s%s%s%s%s" // Optional args: LC, NOR, JI, NL & CMB
+      "%s%s%s%s%s%s%s%s%s%s" // Optional args: LC, NOR, JI, NL, CMB, LT
       " %s\n";           // Assembly level
    printf(format,
           args[MSH], args[POR], args[QOR], args[MID], args[TID], args[LS],
@@ -561,6 +606,8 @@ static void dump_args(int id, const char *args[])
           args[NL][0] == '1' ? "" : args[NL],
           args[CMB][0] == '0' ? "" : " -cmb ",
           args[CMB][0] == '0' ? "" : args[CMB],
+          args[LT][0] == '0' ? "" : " -lt ",
+          args[LT][0] == '0' ? "" : args[LT],
           // Assembly level
           args[ALV]);
    fflush(0);
@@ -578,15 +625,17 @@ static void tmop_require(int id, const char *args[])
    REQUIRE(res[0].diag == MFEM_Approx(res[1].diag));
 }
 
+static constexpr size_t sz = 16;
+
 static inline const char *itoa(const int i, char *buf)
 {
-   std::sprintf(buf, "%d", i);
+   std::snprintf(buf, sz, "%d", i);
    return buf;
 }
 
-static inline const char *dtoa(const double d, char *buf)
+static inline const char *dtoa(const real_t d, char *buf)
 {
-   std::sprintf(buf, "%.4f", d);
+   std::snprintf(buf, sz, "%.4f", d);
    return buf;
 }
 
@@ -605,8 +654,9 @@ public:
       int max_lin_iter  = 100;
       int combo = 0;
       bool normalization = false;
-      double lim_const = 0.0;
-      double jitter = 0.0;
+      real_t lim_const = 0.0;
+      int lim_type = 0;
+      real_t jitter = 0.0;
       set order = {1,2,3,4};
       set target_id = {1,2,3};
       set metric_id = {1,2};
@@ -622,8 +672,9 @@ public:
       Args &LINEAR_ITERATIONS(const int arg) { max_lin_iter = arg; return *this; }
       Args &CMB(const int arg) { combo = arg; return *this; }
       Args &NORMALIZATION(const bool arg) { normalization = arg; return *this; }
-      Args &LIMITING(const double arg) { lim_const = arg; return *this; }
-      Args &JI(const double arg) { jitter = arg; return *this; }
+      Args &LIMITING(const real_t arg) { lim_const = arg; return *this; }
+      Args &LIMIT_TYPE(const int arg) { lim_type = arg; return *this; }
+      Args &JI(const real_t arg) { jitter = arg; return *this; }
 
       Args &POR(set arg) { order = arg; return *this; }
       Args &TID(set arg) { target_id = arg; return *this; }
@@ -633,15 +684,15 @@ public:
       Args &NL(set arg) { newton_loop = arg; return *this; }
    };
    const char *name, *mesh;
-   int NEWTON_ITERATIONS, REFINE, LINEAR_ITERATIONS, COMBO;
+   int NEWTON_ITERATIONS, REFINE, LINEAR_ITERATIONS, COMBO, LIMIT_TYPE;
    bool NORMALIZATION;
-   double LIMITING, JITTER;
+   real_t LIMITING, JITTER;
    set P_ORDERS, TARGET_IDS, METRIC_IDS, Q_ORDERS, LINEAR_SOLVERS, NEWTON_LOOPS;
 public:
    Launch(Args a = Args()):
       name(a.name), mesh(a.mesh),
       NEWTON_ITERATIONS(a.newton_iter), REFINE(a.rs_levels),
-      LINEAR_ITERATIONS(a.max_lin_iter), COMBO(a.combo),
+      LINEAR_ITERATIONS(a.max_lin_iter), COMBO(a.combo), LIMIT_TYPE(a.lim_type),
       NORMALIZATION(a.normalization), LIMITING(a.lim_const), JITTER(a.jitter),
       P_ORDERS(a.order), TARGET_IDS(a.target_id), METRIC_IDS(a.metric_id),
       Q_ORDERS(a.quad_order), LINEAR_SOLVERS(a.lin_solver),
@@ -652,42 +703,44 @@ public:
    {
       if ((id==0) && name) { mfem::out << "[" << name << "]" << std::endl; }
       DEFAULT_ARGS;
-      char ni[8] {}, rs[8] {}, li[8] {}, lc[16] {}, ji[16] {}, cmb[8] {};
+      char ni[sz] {}, rs[sz] {}, li[sz] {}, lc[sz] {}, ji[sz] {},
+           cmb[sz] {}, lt[sz] {};
       args[MSH] = mesh;
       args[RS] = itoa(REFINE,rs);
       args[NI] = itoa(NEWTON_ITERATIONS,ni);
       args[LI] = itoa(LINEAR_ITERATIONS,li);
       args[CMB] = itoa(COMBO,cmb);
       args[LC] = dtoa(LIMITING,lc);
+      args[LT] = itoa(LIMIT_TYPE,lt);
       args[JI] = dtoa(JITTER,ji);
       args[NOR] = NORMALIZATION ? "1" : "0";
       for (int p : P_ORDERS)
       {
-         char por[2] {};
+         char por[sz] {};
          args[POR] = itoa(p, por);
          for (int t : TARGET_IDS)
          {
-            char tid[2] {};
+            char tid[sz] {};
             args[TID] = itoa(t, tid);
             for (int m : METRIC_IDS)
             {
-               char mid[4] {};
+               char mid[sz] {};
                args[MID] = itoa(m, mid);
                for (int q : Q_ORDERS)
                {
                   if (q <= p) { continue; }
-                  char qor[2] {};
+                  char qor[sz] {};
                   args[QOR] = itoa(q, qor);
                   for (int ls : LINEAR_SOLVERS)
                   {
                      // skip some linear solver & metric combinations
                      // that lead to non positive definite operators
                      if (ls == 1 && m != 1) { continue; }
-                     char lsb[2] {};
+                     char lsb[sz] {};
                      args[LS] = itoa(ls, lsb);
                      for (int n : NEWTON_LOOPS)
                      {
-                        char nl[2] {};
+                        char nl[sz] {};
                         args[NL] = itoa(n, nl);
                         tmop_require(id, args);
                         if (!nr) { break; }
@@ -708,13 +761,16 @@ public:
 // id: MPI rank, nr: launch all non-regression tests
 static void tmop_tests(int id = 0, bool all = false)
 {
-#if defined(MFEM_TMOP_MPI) && defined(HYPRE_USING_GPU)
-   cout << "\nAs of mfem-4.3 and hypre-2.22.0 (July 2021) this unit test\n"
-        << "is NOT supported with the GPU version of hypre.\n\n";
-   return;
+#if defined(MFEM_TMOP_MPI)
+   if (HypreUsingGPU())
+   {
+      cout << "\nAs of mfem-4.3 and hypre-2.22.0 (July 2021) this unit test\n"
+           << "is NOT supported with the GPU version of hypre.\n\n";
+      return;
+   }
 #endif
 
-   const double jitter = 1./(M_PI*M_PI);
+   const real_t jitter = 1./(M_PI*M_PI);
 
    Launch(Launch::Args("TC_IDEAL_SHAPE_UNIT_SIZE_2D_KERNEL").
           MESH("../../data/star.mesh").REFINE(1).JI(jitter).
@@ -726,6 +782,19 @@ static void tmop_tests(int id = 0, bool all = false)
           POR({1,2}).QOR({2,3}).
           TID({3}).MID({2})).Run(id,all);
 
+   Launch(Launch::Args("TC_GIVEN_SHAPE_AND_SIZE_2D_KERNEL").
+          MESH("../../data/star.mesh").REFINE(1).JI(jitter).
+          NORMALIZATION(true).
+          POR({1,2}).QOR({2,3}).
+          TID({8}).MID({94}).LS({3})).Run(id,all);
+
+   Launch(Launch::Args("TC_GIVEN_SHAPE_AND_SIZE_3D_KERNEL").
+          MESH("../../data/toroid-hex.mesh").
+          LIMITING(M_PI).LIMIT_TYPE(1).REFINE(1).JI(jitter).
+          NORMALIZATION(true).
+          POR({2}).QOR({4}).
+          TID({8}).MID({338}).LS({3})).Run(id,all);
+
    Launch(Launch::Args("TC_IDEAL_SHAPE_UNIT_SIZE_3D_KERNEL").
           MESH("../../miniapps/meshing/cube.mesh").REFINE(1).JI(jitter).
           POR({1,2}).QOR({2,3}).
@@ -751,7 +820,7 @@ static void tmop_tests(int id = 0, bool all = false)
           NORMALIZATION(true).
           POR({1,2}).QOR({4,6}).
           LINEAR_ITERATIONS(150).
-          TID({5}).MID({80}).LS({3})).Run(id,all);
+          TID({5}).MID({80,94}).LS({3})).Run(id,all);
 
    Launch(Launch::Args("Blade").
           MESH("../../miniapps/meshing/blade.mesh").
@@ -770,6 +839,12 @@ static void tmop_tests(int id = 0, bool all = false)
           POR({1,2}).QOR({2,4}).
           TID({1,2,3}).MID({2})).Run(id,all);
 
+   Launch(Launch::Args("Blade + limiting_expo + normalization").
+          MESH("../../miniapps/meshing/blade.mesh").
+          NORMALIZATION(true).LIMITING(M_PI).LIMIT_TYPE(1).
+          POR({1,2}).QOR({2,4}).
+          TID({1,2,3}).MID({2})).Run(id,all);
+
    Launch(Launch::Args("Cube").
           MESH("../../miniapps/meshing/cube.mesh").REFINE(1).JI(jitter).
           POR({1,2}).QOR({2,4}).
@@ -785,7 +860,7 @@ static void tmop_tests(int id = 0, bool all = false)
           MESH("../../miniapps/meshing/cube.mesh").
           NORMALIZATION(true).
           POR({1,2}).QOR({4,2}).
-          TID({5}).MID({332})).Run(id,all);
+          TID({5}).MID({332,338})).Run(id,all);
 
    // Note: order 1 has no interior nodes, so all residuals are zero and the
    // Newton iteration exits immediately.
@@ -806,6 +881,12 @@ static void tmop_tests(int id = 0, bool all = false)
           POR({1,2}).QOR({2,4}).
           TID({1,2}).MID({321})).Run(id,all);
 
+   Launch(Launch::Args("Toroid-Hex + limiting_expo + norm.").
+          MESH("../../data/toroid-hex.mesh").
+          LIMITING(M_PI).LIMIT_TYPE(1).NORMALIZATION(true).
+          POR({1,2}).QOR({2,4}).
+          TID({1,2}).MID({321})).Run(id,all);
+
    // -m cube.mesh -rs 1 -tid 5 -mid 321 -ni 5 -ls 3 -li 100 -lc 1.0 -nor
    Launch(Launch::Args("Cube + Blast options").
           MESH("../../miniapps/meshing/cube.mesh").REFINE(1).
@@ -856,34 +937,41 @@ static void tmop_tests(int id = 0, bool all = false)
           TID({5}).MID({2})).Run(id,true);
 }
 
-#if defined(MFEM_TMOP_MPI)
-#ifndef MFEM_TMOP_DEVICE
+#ifdef MFEM_TMOP_MPI
 TEST_CASE("tmop_pa", "[TMOP_PA], [Parallel]")
 {
    tmop_tests(Mpi::WorldRank(), launch_all_non_regression_tests);
 }
 #else
-TEST_CASE("tmop_pa", "[TMOP_PA], [Parallel]")
-{
-   Device device;
-   device.Configure(MFEM_TMOP_DEVICE);
-   device.Print();
-   tmop_tests(Mpi::WorldRank(), launch_all_non_regression_tests);
-}
-#endif
-#else
-#ifndef MFEM_TMOP_DEVICE
 TEST_CASE("tmop_pa", "[TMOP_PA]")
 {
    tmop_tests(0, launch_all_non_regression_tests);
 }
-#else
-TEST_CASE("tmop_pa", "[TMOP_PA]")
+#endif
+
+int main(int argc, char *argv[])
 {
-   Device device;
-   device.Configure(MFEM_TMOP_DEVICE);
+#ifdef MFEM_USE_SINGLE
+   std::cout << "\nThe TMOP unit tests are not supported in single"
+             " precision.\n\n";
+   return MFEM_SKIP_RETURN_VALUE;
+#endif
+
+#ifdef MFEM_TMOP_MPI
+   mfem::Mpi::Init();
+   mfem::Hypre::Init();
+#endif
+#ifdef MFEM_TMOP_DEVICE
+   Device device(MFEM_TMOP_DEVICE);
+#else
+   Device device("cpu"); // make sure hypre runs on CPU, if possible
+#endif
    device.Print();
-   tmop_tests(0, launch_all_non_regression_tests);
+
+#ifdef MFEM_TMOP_MPI
+   return RunCatchSession(argc, argv, {"[Parallel]"}, Root());
+#else
+   // Exclude parallel tests.
+   return RunCatchSession(argc, argv, {"~[Parallel]"});
+#endif
 }
-#endif
-#endif
