@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2022, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -15,6 +15,12 @@
 
 int main(int argc, char *argv[])
 {
+#ifdef MFEM_USE_SINGLE
+   std::cout << "\nThe serial unit tests are not supported in single"
+             " precision.\n\n";
+   return MFEM_SKIP_RETURN_VALUE;
+#endif
+
    // Exclude parallel tests.
    return RunCatchSession(argc, argv, {"~[Parallel]"});
 }
